/*---------------------------------------------------------\
|                                                          |
| view.c                                                   |
|                                                          |
| by Joel Beaudoin                                         |
|                                                          |
|----------------------------------------------------------|
|                                                          |
| This module contains data and functions that pertain to  |
| the rendered view of the level in the client area of the |
| window.                                                  |
|                                                          |
\---------------------------------------------------------*/

#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include "BspFileStructs.h"
#include "leveldata.h"

void ChangeSize(GLsizei w, GLsizei h)
{
	GLfloat fAspect;

	// Prevent a divide by zero
	if(h == 0)
		h = 1;

	// Set Viewport to window dimensions
    glViewport(0, 0, w, h);

	fAspect = (GLfloat)w/(GLfloat)h;

	// Reset coordinate system
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	// Produce the perspective projection
	gluPerspective(60.0f, fAspect, 1.0, 10000.0);

	glMatrixMode(GL_MODELVIEW);
	//glLoadIdentity();
}

/*-----------------------------------------------------------------*/

void RenderScene(void)
{
	
	BSPFACE* pCurFace = NULL;
	BSPEDGE* pCurEdge = NULL;
	BSPVERTEX* pCurVertex = NULL;
	long edgeIndex;
	int i, j;

	// Clear the window with current clearing color
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	for(i=0; i<levelModel.numFaces; i++)
	{
		pCurFace = &faces[levelModel.idxFaces + i];
		glColor3f(v[i].x, v[i].y, v[i].z);
		glBegin(GL_POLYGON);
			for(j=0; j<pCurFace->numEdges; j++)
			{
				edgeIndex = edgeIndices[pCurFace->idxEdges + j];
				if(edgeIndex > 0)
				{
					pCurEdge = &edges[edgeIndex];
					pCurVertex = &vertices[pCurEdge->idxV0];
				}
				else
				{
					pCurEdge = &edges[-edgeIndex];
					pCurVertex = &vertices[pCurEdge->idxV1];
				}
				glVertex3f(pCurVertex->x, pCurVertex->z, -pCurVertex->y);
			}
		glEnd();
	}
	// Flush drawing commands
	glFlush();
}